@@ -5,7 +5,7 @@
 #
 # Copyright (c) 2008-2009 Yu Yuwei <acevery@gmail.com>
 # Copyright (c) 2009-2014 Caius "kaio" CHANCE <me@kaio.net>
-# Copyright (c) 2012-2018 Mike FABIAN <mfabian@redhat.com>
+# Copyright (c) 2012-2021 Mike FABIAN <mfabian@redhat.com>
 # Copyright (c) 2019      Peng Wu <alexepico@gmail.com>
 #
 # This library is free software; you can redistribute it and/or
@@ -18,23 +18,22 @@
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 # Lesser General Public License for more details.
 #
-# You should have received a copy of the GNU Lesser General Public
-# License along with this library; if not, write to the Free Software
-# Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
-#
+# You should have received a copy of the GNU Lesser General Public License
+# along with this library.  If not, see <http://www.gnu.org/licenses/>
 '''
 Module for ibus-table to access the sqlite3 databases
 '''
 import os
 import os.path as path
-import sys
 import shutil
 import sqlite3
 import uuid
 import time
 import re
 import logging
+import json
 import chinese_variants
+import ibus_table_location
 
 LOGGER = logging.getLogger('ibus-table')
 
@@ -246,30 +245,10 @@ class TabSqliteDb:
                 + 'is it an outdated database?')
             self.user_can_define_phrase = False
 
-        self.dynamic_adjust = self.ime_properties.get('dynamic_adjust')
-        if self.dynamic_adjust:
-            self.dynamic_adjust = bool(self.dynamic_adjust.lower() == u'true')
-        else:
-            print(
-                'Could not find "dynamic_adjust" entry from database, '
-                + 'is it an outdated database?')
-            self.dynamic_adjust = False
-
-        self.suggestion_mode = self.ime_properties.get('suggestion_mode')
-        if self.suggestion_mode:
-            self.suggestion_mode = bool(
-                self.suggestion_mode.lower() == u'true')
-        else:
-            print(
-                'Could not find "suggestion_mode" entry from database, '
-                + 'is it an outdated database?')
-            self.suggestion_mode = False
-
         self.rules = self.get_rules()
         self.possible_tabkeys_lengths = self.get_possible_tabkeys_lengths()
         self.startchars = self.get_start_chars()
 
-        import ibus_table_location
         tables_path = path.join(ibus_table_location.data_home(), 'tables')
         cache_name = os.path.basename(self.filename).replace('.db', '.cache')
         self.cache_path = path.join(tables_path, cache_name)
@@ -357,8 +336,7 @@ class TabSqliteDb:
                                 self.get_number_of_columns_of_phrase_table(
                                     user_db))
                             self.old_phrases = None
-                        from time import strftime
-                        timestamp = strftime('-%Y-%m-%d_%H:%M:%S')
+                        timestamp = time.strftime('-%Y-%m-%d_%H:%M:%S')
                         LOGGER.debug(
                             'Renaming the incompatible database to "%s".',
                             user_db+timestamp)
@@ -369,7 +347,7 @@ class TabSqliteDb:
                         if os.path.exists(user_db+'-wal'):
                             os.rename(user_db+'-wal', user_db+'-wal'+timestamp)
                         LOGGER.debug(
-                            'Creating a new, empty database "s".', user_db)
+                            'Creating a new, empty database "%s".', user_db)
                         self.init_user_db(user_db)
                         LOGGER.debug(
                             'If user phrases were successfully recovered from '
@@ -400,8 +378,7 @@ class TabSqliteDb:
             ''' % user_db)
         except:
             LOGGER.debug('Could not open the database %s.', user_db)
-            from time import strftime
-            timestamp = strftime('-%Y-%m-%d_%H:%M:%S')
+            timestamp = time.strftime('-%Y-%m-%d_%H:%M:%S')
             LOGGER.debug('Renaming the incompatible database to "%s".',
                          user_db+timestamp)
             if os.path.exists(user_db):
@@ -511,7 +488,6 @@ class TabSqliteDb:
         if DEBUG_LEVEL > 1:
             LOGGER.debug('load_phrases_cache()')
         try:
-            import json
             self._phrases_cache = json.load(open(self.cache_path, 'r'))
             snum = self._phrases_cache.get('serial_number')
             if not snum or (snum != self._snum):
@@ -534,7 +510,6 @@ class TabSqliteDb:
         if DEBUG_LEVEL > 1:
             LOGGER.debug('save_phrases_cache()')
         try:
-            import json
             self._phrases_cache['serial_number'] = self._snum
             _cache_path = self.cache_path + '.tmp'
             # The system may be break stores on rebooting, so
@@ -648,7 +623,8 @@ class TabSqliteDb:
         '''
         select_sqlstr = 'SELECT val from main.ime WHERE attr = :attr'
         update_sqlstr = 'UPDATE main.ime SET val = :val WHERE attr = :attr;'
-        insert_sqlstr = 'INSERT INTO main.ime (attr, val) VALUES (:attr, :val);'
+        insert_sqlstr = (
+            'INSERT INTO main.ime (attr, val) VALUES (:attr, :val);')
         for attr, val in attrs:
             sqlargs = {'attr': attr, 'val': val}
             if self.db.execute(select_sqlstr, sqlargs).fetchall():
@@ -750,8 +726,6 @@ class TabSqliteDb:
     def get_no_check_chars(self):
         '''Get the characters which engine should not change freq'''
         _chars = self.ime_properties.get('no_check_chars')
-        if type(_chars) != type(u''):
-            _chars = _chars.decode('utf-8')
         return _chars
 
     def add_phrases(self, phrases, database='main'):
@@ -1038,7 +1012,7 @@ class TabSqliteDb:
     def select_words(
             self, tabkeys=u'', onechar=False, chinese_mode=4,
             single_wildcard_char=u'', multi_wildcard_char=u'',
-            auto_wildcard=False):
+            auto_wildcard=False, dynamic_adjust=False):
         '''
         Get matching phrases for tabkeys from the database.
         '''
@@ -1053,7 +1027,7 @@ class TabSqliteDb:
             # for some users really like to select only single characters
             one_char_condition = ' AND length(phrase)=1 '
 
-        if self.user_can_define_phrase or self.dynamic_adjust:
+        if self.user_can_define_phrase or dynamic_adjust:
             sqlstr = '''
             SELECT tabkeys, phrase, freq, user_freq FROM
             (
@@ -1357,7 +1331,8 @@ class TabSqliteDb:
         table code of the last character in the phrase.
 
         Let’s assume the phrase is “天下大事”. The goucima (構詞碼
-        = “word formation keys”) for these 4 characters are:
+        = “word formation keys”) for these 4 characters when
+        using the wubi-jidian86 table are:
 
             character goucima
             天        gdi
@@ -1379,8 +1354,6 @@ class TabSqliteDb:
         '''
         if DEBUG_LEVEL > 1:
             LOGGER.debug('phrase=%s rules%s', phrase, self.rules)
-        if type(phrase) != type(u''):
-            phrase = phrase.decode('UTF-8')
         # Shouldn’t this function try first whether the system database
         # already has an entry for this phrase and if yes return it
         # instead of constructing a new entry according to the rules?
@@ -1469,7 +1442,7 @@ class TabSqliteDb:
             return result[0][0]
         return 0
 
-    def check_phrase(self, tabkeys=u'', phrase=u''):
+    def check_phrase(self, tabkeys=u'', phrase=u'', dynamic_adjust=False):
         '''Adjust user_freq in user database if necessary.
 
         Also, if the phrase is not in the system database, and it is a
@@ -1479,15 +1452,11 @@ class TabSqliteDb:
         '''
         if DEBUG_LEVEL > 1:
             LOGGER.debug('tabkey=%s phrase=%s', tabkeys, phrase)
-        if type(phrase) != type(u''):
-            phrase = phrase.decode('utf8')
-        if type(tabkeys) != type(u''):
-            tabkeys = tabkeys.decode('utf8')
         if not tabkeys or not phrase:
             return
         if self._is_chinese and phrase in CHINESE_NOCHECK_CHARS:
             return
-        if not self.dynamic_adjust:
+        if not dynamic_adjust:
             if not self.user_can_define_phrase or not self.is_chinese:
                 return
             tabkeys = self.parse_phrase(phrase)
@@ -1544,8 +1513,6 @@ class TabSqliteDb:
         wqi	你	1490000000
         wqiy	你	1490000000
         '''
-        if type(phrase) != type(u''):
-            phrase = phrase.decode('utf8')
         sqlstr = '''
         SELECT tabkeys FROM main.phrases WHERE phrase = :phrase
         ORDER by length(tabkeys) ASC;
@@ -1559,6 +1526,8 @@ class TabSqliteDb:
             self, tabkeys=u'', phrase=u'', database='user_db', commit=True):
         '''Remove phrase from database
         '''
+        LOGGER.info('Removing tabkeys=%s, phrase=%s, database=%s commit=%s',
+                    tabkeys, phrase, database, commit)
         if not phrase:
             return
         if tabkeys:
@@ -1577,6 +1546,21 @@ class TabSqliteDb:
             self.db.commit()
         self.invalidate_phrases_cache(tabkeys)
 
+    def remove_all_phrases_from_user_db(self) -> None:
+        '''
+        Remove all phrases from the user database, i.e. delete all the
+        data learned from user input.
+        '''
+        LOGGER.info('Removing all phrases from the user database.')
+        try:
+            self.db.execute('DELETE FROM user_db.phrases;')
+            self.db.commit()
+            self.db.execute('PRAGMA wal_checkpoint;')
+            self.reset_phrases_cache()
+        except Exception:
+            LOGGER.exception(
+                'Unexpected error removing all phrases from database.')
+
     def extract_user_phrases(
             self, database_file='', old_database_version='0.0'):
         '''extract user phrases from database'''
